# Notes:
# - works on R 2.8.0
# - libraries are for HUMAN genes

# Set Input Variables
# - Exp.Gene: genes to be tested for enrichment
# - All.Gene: all genes that can be retrieved by the experiment 
#             (e.g. all the ones on the array)

# Load Libraries (human)

library (GO.db)
library (KEGG.db)
library (org.Hs.eg.db)

# Data prep on libraries

GOTERM.ls  <- as.list (GOTERM)
GOTERM.chv <- unlist (lapply (GOTERM.ls, Term))
names (GOTERM.chv) <- names (GOTERM.ls)
GO2EG.ls   <- as.list (org.Hs.egGO2ALLEGS)

KEGGTERM.chv <- unlist (as.list (KEGGPATHID2NAME))
KEGG2EG.ls   <- as.list (org.Hs.egPATH2EG)

k1.n <- length (KEGG2EG.ls)
k2.n <- length (KEGGTERM.chv)

names (KEGG2EG.ls) <- 
	paste (
		"hsa",
		names (KEGG2EG.ls),
		sep = ""
	)

names (KEGGTERM.chv) <- 
	paste (
		"hsa",
		names (KEGGTERM.chv),
		sep = ""
	)

rm (k1.n, k2.n, GOTERM.ls)

## !Test!
## query = Regulation of Cell Cycle
## universe = all GO and KEGG genes

Exp.Gene <- 
	unique (GO2EG.ls[["GO:0051726"]])
All.Gene <- 
	unique (
		c (
			unlist (GO2EG.ls),
			unlist (KEGG2EG.ls)
		)
	)


# Pre-filter libraries
# - GO (Gene Ontology)
# - KEGG (pathways)

# parameter explanation
# - tG_min.n: set around 7-15 (10)
# - tG_max.n: set around 400-600 (500)
# - tE_min.n: set around 5-15 (10)
# * in case of limited genomic coverage, 
#   set 'tE_min.n' to 5, but keep 'tG_min.n' on 10
# * in case of limited results after p-value correction,
#   increase 'tE_min.n' and decrease 'tG_max.n'

tG_min.n <-  10
tE_min.n <-  10
tG_max.n <- 500

GO2EG_f.ls <- 
	f.enr.prefilter (
		GO.list       = GO2EG.ls, 
                sample.Entrez = Exp.Gene, 
                thrGN.min.n   = tG_min.n, 
		thrGN.max.n   = tG_max.n, 
		thrEN.n       = tE_min.n)

KEGG2EG_f.ls <- 
	f.enr.prefilter (
		GO.list       = KEGG2EG.ls, 
                sample.Entrez = Exp.Gene, 
                thrGN.min.n   = tG_min.n, 
		thrGN.max.n   = tG_max.n, 
		thrEN.n       = tE_min.n)

rm (tG_min.n, tE_min.n, tG_max.n)

# Merge KEGG and GO gene-sets

GS2EG_f.ls   <- c (KEGG2EG_f.ls, GO2EG_f.ls)
GSTERM_f.chv <- 
	c (	
		KEGGTERM.chv[names (KEGG2EG_f.ls)],
		GOTERM.chv[names (GO2EG_f.ls)]
	)


# Compute Fisher's Test and other enrichment stats

enr.df <- f.Fisher.enr (
			GOf.list      = GS2EG_f.ls, 
			GOterm.chv    = GSTERM_f.chv,
			sample.Entrez = Exp.Gene, 
			uni.Entrez    = All.Gene)

# Correct p-values

c.n <- ncol (enr.df)
adjusted.nv <- p.adjust (enr.df$p.val, method = "BH")

enr.df <- cbind (enr.df, adjusted.nv)
colnames (enr.df)[c.n + 1] <- "adj.pval"

# Final filter and write

# - set the p-value threshold to 0.005, or 0.01, or 0.05
#   depending on the desired stringency

tPval.n <- 0.01

enr_f.df <- enr.df[enr.df$adj.pval < tPval.n, ]

# - set the file name

file.name <- "EnrRes_01.txt"

write.table (
	enr_f.df, 
	file = file.name, 
	sep = "\t", 
	quote = F, 
	row.names = F)

# Formatting for Enrichment Map Cytoscape plugin
# ('generic' format)

ph.n = + 1

r.n <- nrow (enr.df)

enr_EM_1_1.df <- enr.df[, c ("GO.ID", "GO.term", "adj.pval")]
enr_EM_1_2.df <- 
	as.data.frame (
		cbind (
			rep (0, r.n),
			rep (ph.n, r.n)
		)
	)
colnames (enr_EM_1_2.df) <- c ("FDR", "Phenotype")

enr_EM_1.df <- cbind (enr_EM_1_1.df, enr_EM_1_2.df)

# do the enrichment resulting from down-regulation, 
# or the corresponding switched comparison 
# (in any case, the complementary Exp.Gene set),
# but with: ph.n = - 1, 
# in order to generate enr_EM_2.df

enr_EM.df <- cbind (enr_EM_1.df, enr_EM_2.df)

rm (enr_EM_1.df, enr_EM_2.df, enr_EM_1_1.df, enr_EM_1_2.df, enr_EM_2_1.df, enr_EM_2_2.df)

file.name <- "EnrRes_EM.txt"

write.table (
	enr_EM.df, 
	file = file.name, 
	sep = "\t", 
	quote = F, 
	row.names = F)
